#####################################
#
# Jeu Hexapion
# ---------------
#
# Auteur: Franck Lagrave
# Mars 2022
# Lycée Beaussier - TraAM Nice -> IA
#
#####################################

import tkinter as tk
from PIL import Image, ImageTk
from tkinter.messagebox import askquestion, showwarning, showinfo  # IHM
from random import choice
from apprentissage import index_plateau, arbre_boite

EMPTY = 0
PAWN_W = 1
PAWN_B = 2

NBL, NBC = 4, 4
SIDE = 100
COLORS = ['Cornsilk', 'Sienna']

STATE_NONE = 0
STATE_SHOW_MOVES = 1
STATE_WON = 2

PUZZLES = [[(3, 3), [(2, 0), (2, 1), (2, 2)], [(0, 0), (0, 1), (0, 2)]],
           [(3, 4),
           [(2, 0), (2, 1), (2, 2), (2, 3)], [(0, 0), (0, 1), (0, 2), (0, 3)]]]

jeu = {"niv": [],
       "coups": 0,
       "pions": [PAWN_W, PAWN_B],
       "pion": PAWN_W,
       "mov": False,
       "ordi": True,
       "ia": False,
       "verrou": True}

joueurs = {PAWN_W: "blancs", PAWN_B: "noirs"}

Boite = {"Billes": []}
Indices = [""]

# Boite["8801"] = [((0,1),(1,1)),((0,1),(1,0)),((0,2),(1,2))]
# Boite["7397"] = [((0,0),(1,0)),((0,0),(1,1)),((0,2),(1,2)),((0,2),(1,1))]


class Engine():
    def __init__(self, parent, *args, **Kwargs):
        self.app = parent
        self.tries = 0
        self.joueurs = "Au tour des " + joueurs[jeu["pion"]]
        self.init_board(NBL, NBC)
        self.moves = []

    def init_board(self, nbl, nbc):
        self.nbl = nbl
        self.nbc = nbc
        self.board = [[0]*self.nbc for i in range(self.nbl)]

    def set_board(self, puzzle):
        """Initialise le plateau suivant les parametres de <puzzle>:
        [[white pawn sq], [black pawn sq] ]
        """
        self.moves = []
        self.init_board(puzzle[0][0], puzzle[0][1])
        jeu["niv"] = puzzle
        for piece in [PAWN_W, PAWN_B]:
            for l, c in puzzle[piece]:
                self.board[l][c] = piece

    def move(self, start, end):
        ls, cs = start
        le, ce = end
        piece = self.board[ls][cs]
        goal = self.board[le][ce]
        self.board[ls][cs] = EMPTY
        self.board[le][ce] = piece
        self.inc_tries(1)
        self.moves.append((end))
        jeu["mov"] = True

    def move_list(self, start):
        lmove = []
        ls, cs = start
        piece = self.board[ls][cs]

        if piece == PAWN_W:
            if 0 <= ls-1 and self.board[ls-1][cs] == EMPTY:
                lmove.append((ls-1, cs))
            if 0 <= ls-1 and cs+1 < self.nbc\
               and self.board[ls-1][cs+1] == PAWN_B:
                lmove.append((ls-1, cs+1))
            if 0 <= ls-1 and 0 <= cs-1 and self.board[ls-1][cs-1] == PAWN_B:
                lmove.append((ls-1, cs-1))
        elif piece == PAWN_B:
            if ls+1 < self.nbl and self.board[ls+1][cs] == EMPTY:
                lmove.append((ls+1, cs))
            if ls+1 < self.nbl and cs+1 < self.nbc\
               and self.board[ls+1][cs+1] == PAWN_W:
                lmove.append((ls+1, cs+1))
            if ls+1 < self.nbl and 0 <= cs-1\
               and self.board[ls+1][cs-1] == PAWN_W:
                lmove.append((ls+1, cs-1))
        return lmove

    def inc_tries(self, inc):
        self.tries += inc
        jeu["coups"] = self.tries
        jeu["pion"] = jeu["pions"][self.tries % 2]
        # pair -> White / impair -> Black
        self.joueurs = "Au tour des " + joueurs[jeu["pion"]]


class Board(tk.Frame):
    def __init__(self, parent, *args, **kwargs):
        tk.Frame.__init__(self, parent, *args, **kwargs)
        self.app = parent
        self.side = SIDE
        self.engine = parent.engine
        self.state = STATE_NONE
        self.piece = EMPTY
        self.drawn_piece = []
        self.width = self.engine.nbc*SIDE
        self.height = self.engine.nbl*SIDE
        self.board = tk.Canvas(self,
                               width=self.width,
                               height=self.height,
                               bg='white')
        self.init_game()
        self.board.pack(fill="both", expand=True)
        self.board.bind("<Button-1>", self.click)

    def init_game(self):
        self.width = self.engine.nbc*SIDE
        self.height = self.engine.nbl*SIDE
        self.board.config(width=self.width, height=self.height)
        self.grid = [[] for i in range(self.engine.nbl)]
        for i in range(self.engine.nbl):
            for j in range(self.engine.nbc):
                self.grid[i].append(
                    self.board.create_rectangle(
                        j*self.side+1,
                        i*self.side+1, (j+1)*self.side+1, (i+1)*self.side+1,
                        fill=COLORS[(i+j) % 2]))

    def delete_board(self):
        for line in self.grid:
            for square in line:
                self.board.delete(square)
        for p in self.drawn_piece:
            self.board.delete(p)

    def draw_piece(self, piece, l, c):
        self.drawn_piece.append(
            self.board.create_image(
                (c+0.5)*SIDE, (l+0.5)*SIDE, image=self.app.images[piece]))

    def draw_game(self):
        self.delete_board()
        self.init_game()
        for i in range(self.engine.nbl):
            for j in range(self.engine.nbc):
                if self.engine.board[i][j] != EMPTY:
                    self.draw_piece(self.engine.board[i][j], i, j)
        # self.app.menu.var_tries.set(str(self.engine.tries))
        self.app.menu.var_joueurs.set(str(self.engine.joueurs))
        self.app.menu.var_moves.set(str(self.engine.moves))

    def click(self, event):
        l, c = self.square_num(event)
        board = self.engine.board
        piece = board[l][c]
        if jeu["verrou"]:
            if self.state == STATE_NONE:
                if piece != EMPTY and piece != jeu["pion"]:
                    showwarning('Erreur', "Ce n'est pas à vous de jouer")
                if piece != EMPTY and piece == jeu["pion"]:
                    lmove = self.engine.move_list((l, c))
                    self.change_color(lmove, 'Lightgreen')
                    self.state = STATE_SHOW_MOVES

            else:  # elif self.state == STATE_SHOW_MOVES:
                if self.piece in [PAWN_W, PAWN_B]:
                    lmove = self.engine.move_list(self.prev_click)
                    if (l, c) in lmove:
                        self.engine.move(self.prev_click, (l, c))

                    self.draw_game()
                    self.state = STATE_NONE
                    self.verification(l, c)
                if jeu["ordi"] and jeu["pion"] == PAWN_B:
                    ind = self.index()
                    Indices.append(ind)
                    if ind not in Boite:
                        Boite[ind] = arbre_boite(ind, self.engine.nbc)
                    # print(Boite)
                    bille = choice(Boite[self.index()])
                    Boite["Billes"].append(bille)
                    # print("Billes",Boite["Billes"])
                    l, c = bille[0]
                    board = self.engine.board
                    piece = board[l][c]
                    self.prev_click = (l, c)
                    l, c = bille[1]
                    self.engine.move(self.prev_click, (l, c))
                    self.draw_game()
                    self.state = STATE_NONE
                    self.verification(l, c)

                jeu["mov"] = False

        self.prev_click = (l, c)
        jeu["prev"] = (l, c)
        self.piece = piece
        jeu["piece"] = piece

    def pion_dame(self, l, c):
        cond1 = (l, c) in jeu["niv"][2] and jeu["pion"] == PAWN_B
        cond2 = (l, c) in jeu["niv"][1] and jeu["pion"] == PAWN_W
        return jeu["mov"] and (cond1 or cond2)

    def pion_bloque(self, l, c):
        return all(
            [self.engine.move_list(start) == []
                for start in [(l, c) for l in range(self.engine.nbl)
                              for c in range(self.engine.nbc)
                              if self.engine.board[l][c] == jeu["pion"]]])

    def pion_manquant(self, l, c):
        return all(
            [self.engine.board[l][c] != jeu["pion"]
                for l in range(self.engine.nbl)
                for c in range(self.engine.nbc)])

    def verification(self, l, c):
        if self.pion_dame(l, c)\
           or self.pion_bloque(l, c) or self.pion_manquant(l, c):
            # self.state = STATE_WON
            jeu["verrou"] = False

            gagnant = jeu["pions"][(self.engine.tries + 1) % 2]
            if jeu["ia"] and gagnant == 1:
                if len(Boite[Indices[-1]]) != 1:
                    Boite[Indices[-1]].remove(Boite["Billes"][-1])
                else:
                    if len(Boite["Billes"]) >= 2:
                        Boite[Indices[-2]].remove(Boite["Billes"][-2])
                    else:
                        if len(Boite["Billes"]) >= 3:
                            Boite[Indices[-3]].remove(Boite["Billes"][-3])
            Boite["Billes"] = []
            r = askquestion('Résultat',
                            f'Bravo les {joueurs[gagnant]} ont gagné.\
                            \nVoulez-vous recommencer ?')
            if r == 'no':
                App.fin()
            else:
                App.nouv_partie()

    def change_color(self, squares, color):
        self.draw_game()
        for l, c in squares:
            self.board.itemconfig(self.grid[l][c], fill=color)

    def square_num(self, event):
        return event.y // SIDE, event.x // SIDE

    def change_side(self, size):
        self.side = size

    # retourner l'index correspondant au plateau actuel
    # int('110001222',3) = 8801 ; int('101010222',3) = 7397
    # def index(self):
    #     index = 0
    #     for i in range(self.engine.nbl):
    #         for j in range(self.engine.nbc):
    #             index += self.engine.board[i][j]*pow(3,
    #                                                  i*self.engine.nbl + j)
    #     return str(int(index))
    def index(self):
        ch = ""
        for i in range(self.engine.nbl):
            for j in range(self.engine.nbc):
                ch += str(self.engine.board[i][j])
        return index_plateau(ch, self.engine.nbc)  # fct eleve


class Menu(tk.Frame):
    def __init__(self, parent, *args, **kwargs):
        tk.Frame.__init__(self, parent, *args, **kwargs)
        self.app = parent
        self.level_nb = 0
        self.engine = parent.engine
        self.board = parent.board
        self.var_level = tk.StringVar()
        # self.var_tries = tk.StringVar()
        self.var_joueurs = tk.StringVar()
        self.var_moves = tk.StringVar()

        self.info = tk.Frame(self)
        # self.tries = tk.Label(self.info, textvariable=self.var_tries,
        #                       font=("Helvetica", 16))
        self.joueurs = tk.Label(self.info, textvariable=self.var_joueurs,
                                font=("Helvetica", 16))
        self.bar = tk.Label(self.info, text=" ... ", font=("Helvetica", 16))
        # text=" / "

        self.moves = tk.Label(self.app, textvariable=self.var_moves,
                              font=("Helvetica", 16))

        self.level = tk.Label(self, textvariable=self.var_level,
                              font=("Helvetica", 16))
        self.level.pack(side='top')

        self.boutons = tk.Frame(self)
        # self.bouton_next = tk.Button(self.boutons, image=self.app.images[3],
        #                              command=self.prev_level)
        # self.bouton_prev = tk.Button(self.boutons, image=self.app.images[4],
        #                              command=self.next_level)

        self.bouton_regles = tk.Button(self.boutons, text="Règles",
                                       command=self.regles)
        self.boutons.pack(side='top')

        self.info.pack(side='top')
        self.bar.pack(side="right", expand=False)
        # self.tries.pack(side="right", expand=False)
        self.joueurs.pack(side="right", expand=False)
        # self.moves.pack(side='bottom',expand=False)

        # self.bouton_next.pack(side="left")
        # self.bouton_prev.pack(side="right")
        self.bouton_regles.pack(side='bottom')  # , padx=3, pady=3)

    def load_level(self):
        pzl = PUZZLES[self.level_nb]
        self.engine.set_board(pzl)
        self.board.state = STATE_NONE
        self.var_level.set("Niveau " + str(self.level_nb + 1))
        self.board.draw_game()

    def next_level(self):
        if self.level_nb < len(PUZZLES)-1:
            self.level_nb += 1
        self.load_level()
        if jeu["ia"]:
            self.new()

    def prev_level(self):
        if self.level_nb > 0:
            self.level_nb -= 1
        self.load_level()
        if jeu["ia"]:
            self.new()

    def apprend(self):
        jeu["ia"] = True

    def stop(self):
        jeu["ia"] = False

    def new(self):
        Boite = {}
        Indices = [""]
        Billes = []
        jeu["ia"] = True

    def regles(self):
        """
            procédure d'affichage des règles du jeu
        """
        showinfo(
            "Règles du jeu hexapion",
            """
            Vous possédez 3 pions se déplaçant comme des pions aux échecs:

            -ils se déplacent vers l'avant
             d'une seule case à chaque tour
            -ils ne peuvent "manger"
             un pion adverse qu'en diagonale

            3 conditions de victoire possibles:

            -vous parvenez à amener un pion
             à l'autre bout du plateau
            -l'adversaire est bloqué
            -l'adversaire n'a plus de pion


            Pour déplacer un pion,
            cliquez sur lui puis sur la case
            où vous souhaitez le placer.
            """
        )


class MainApp(tk.Frame):
    def __init__(self, parent, *args, **kwargs):
        tk.Frame.__init__(self, parent, *args, **kwargs)
        self.root = parent

        self.root.title("Jeu Hexapion")

        self.engine = Engine(self)

        self.img_name = ["pion.png", "pion_noir.png", "left.png", "right.png"]
        self.init_images()

        self.board = Board(self)
        self.board.pack(side="right", padx=10, pady=10, fill="both",
                        expand=True)

        self.menu = Menu(self)
        self.menu.pack(side="left", padx=10, pady=10, fill="both", expand=True)
        self.menu.load_level()

    def init_images(self):
        self.images = {}
        for i in range(len(self.img_name)):
            name = self.img_name[i]
            image = Image.open(name)
            if i in [0, 1]:
                image = image.resize((SIDE, SIDE), Image.ANTIALIAS)
            else:
                image = image.resize((SIDE//4, SIDE//4), Image.ANTIALIAS)
            img = ImageTk.PhotoImage(image)
            self.images[i+1] = img

    def fin(self):
        self.root.destroy()

    def nouv_partie(self):
        jeu["pion"] = PAWN_W
        self.board.engine.tries = 0
        self.board.engine.moves = []
        self.board.engine.joueurs = "Au tour des " + joueurs[jeu["pion"]]
        self.board.init_game()
        self.board.delete_board()
        self.board.draw_game()
        self.menu.load_level()

        jeu["verrou"] = True  # pass

    def IA(self):
        jeu["ia"] = True

    def mainloop(self):
        self.root.mainloop()


if __name__ == '__main__':
    j = 0
    while j not in {1, 2}:
        j = int(input("nb de joueurs 1/2 ? "))
    if j == 2:
        jeu["ordi"] = False
    App = MainApp(tk.Tk())
    if jeu["ordi"]:
        App.IA()
    App.pack(side="top", fill="both", expand=True)
    App.mainloop()
