#
# Fichier à compléter: apprentissage.py
#

EMPTY = 0  # case vide
PAWN_W = 1  # pion blanc
PAWN_B = 2  # pion noir

# Les fonctions suivantes sont données à titre indicatif pour le prof,
# les différents codes élèves seront validés par les jeux de tests
# (non exhaustifs...) fournis pour chaque fonction.


def conv_ternaire(n):
    """int -> str
    Bases de numération : écriture base 10 vers écriture base 3.

    hypothese : n est un entier naturel donné par son écriture décimale.

    renvoie: l'écriture en base 3 de l'entier naturel n
    sous forme d'une chaîne de caractères.
    """
    C = ""
    while n != 0:
        C = str(n % 3) + C
        n = n//3
    return C

# Jeu de tests
assert conv_ternaire(7397) == '101010222'
assert conv_ternaire(8801) == '110001222'


def index_plateau(ch, nbc):
    """str*int -> str
    Calcul de l'index du plateau identifiant une boîte d'allumettes.

    nbc est le nb de colonnes du plateau.
    ch représente la situation du plateau par un parcours des lignes
    de gauche à droite et de haut en bas.

    hypothese: ch est une chaine de caractères de longueur 9 composée
    uniquement de "0","1","2"le nb de "1" le nb de "2" étant au max nbc.

    renvoie: l'index du plateau sous forme d'une chaîne de caractères.
    """
    return str(int(ch[::-1], 3))

# Jeu de tests
assert index_plateau("222100011", 3) == "8801"
assert index_plateau("222010101", 3) == "7397"


def grille(ind, nbc):
    """str*int -> list[list[int]]
    Représentation du plateau identifiant une boîte sous forme d'un tableau.

    nbc est le nb de colonnes du plateau.
    ind est l'index du plateau sous forme d'une chaîne de caractères.

    hypothese: ind est une chaine de caractères représentant l'index plateau.

    renvoie: l'index du plateau sous forme d'un tableau 3 x nbc représentant
    la situation de chaque case modélisée par les entiers 0 1 ou 2.
    """
    plateau = [[0]*nbc for i in range(3)]
    coord = [(i, j) for i in range(3) for j in range(nbc)]
    for i, pion in enumerate(conv_ternaire(int(ind))[::-1]):
        plateau[coord[i][0]][coord[i][1]] = int(pion)
    return plateau

# Jeu de tests
assert grille("8801", 3) == [[2, 2, 2], [1, 0, 0], [0, 1, 1]]
assert grille("7397", 3) == [[2, 2, 2], [0, 1, 0], [1, 0, 1]]


def list_move(start, plateau, lmove):
    """tuple*list[list[int]]*list -> list[tuple(tuple)]
    Détermine les coups possibles pour un pion noir.

    une case du plateau est identifiée par (numero_ligne, numero_colonne)
    (0, 0) est la case en haut à gauche avec un pion noir au début du jeu.
    start est la position de départ d'un pion donnée par un tuple (ls, cs).
    plateau est un tableau 3xnbc représentant la situation de chaque case
    modélisée par les entiers 0 1 ou 2.
    lmove est une liste mutable soit vide soit contenant des tuples de tuples,
    chacun représentant un déplacement possible. ex:((0, 0), (1, 0)) représente
    le déplacement possible du pion noir en (0, 0) vers la case (1, 0).

    hypothese: start est la position d'un pion noir choisi sur le plateau.

    renvoie: liste (mutable) soit vide soit contenant des tuples de tuples,
    représentant tous les déplacements possibles pour le pion noir choisi.
    """
    nbl, nbc = 3, len(plateau[0])
    ls, cs = start
    piece = plateau[ls][cs]
    if ls+1 < nbl and plateau[ls+1][cs] == EMPTY:
        lmove.append(((ls, cs), (ls+1, cs)))
    if ls+1 < nbl and cs+1 < nbc and plateau[ls+1][cs+1] == PAWN_W:
        lmove.append(((ls, cs), (ls+1, cs+1)))
    if ls+1 < nbl and 0 <= cs-1 and plateau[ls+1][cs-1] == PAWN_W:
        lmove.append(((ls, cs), (ls+1, cs-1)))
    return lmove

# Jeu de tests
assert list_move((0, 0), grille("8801", 3), []) == []
assert list_move((0, 1), grille("8801", 3), []) == [((0, 1), (1, 1)),
                                                    ((0, 1), (1, 0))]
assert list_move((0, 2), grille("8801", 3), []) == [((0, 2), (1, 2))]


def arbre_boite(ind, nbc):
    """str*int -> list[tuple(tuple)]
    Détermine les coups possibles pour une boîte d'allumettes.


    nbc est le nb de colonnes du plateau.
    ind est l'index du plateau sous forme d'une chaîne de caractères.

    hypothese: ind est une chaine de caractères représentant l'index plateau.

    renvoie: liste (mutable) soit vide soit contenant des tuples de tuples,
    représentant tous les déplacements possibles pour tous les pions noirs
    (c-à-d les flêches sur les boites d'allumettes).
    """
    nbl = 3
    plateau = grille(ind, nbc)
    lmove = []
    for i in range(nbl):
        for j in range(nbc):
            if plateau[i][j] == PAWN_B:
                list_move((i, j), plateau, lmove)
    return lmove

# Jeu de tests
assert arbre_boite("8801", 3) ==\
    [((0, 1), (1, 1)), ((0, 1), (1, 0)), ((0, 2), (1, 2))]
assert arbre_boite("7397", 3) ==\
    [((0, 0), (1, 0)), ((0, 0), (1, 1)), ((0, 2), (1, 2)), ((0, 2), (1, 1))]
